@@ -19,21 +19,32 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <glib.h>
+#include "config.h"
 
 #include "seahorse-service.h"
 #include "seahorse-key.h"
-#include "seahorse-pgp-key.h"
-#include "seahorse-pgp-operation.h"
 #include "seahorse-gconf.h"
 #include "seahorse-util.h"
 #include "seahorse-libdialogs.h"
 
+#include "pgp/seahorse-pgp-dialogs.h"
+#include "pgp/seahorse-pgp-key.h"
+#include "pgp/seahorse-pgp-operation.h"
+
+#include <string.h>
+
+#include <glib.h>
+#include <glib/gi18n.h>
+
 /* flags from seahorse-service-cyrpto.xml */
 #define FLAG_QUIET 0x01
 
 G_DEFINE_TYPE (SeahorseServiceCrypto, seahorse_service_crypto, G_TYPE_OBJECT);
 
+/* Note that we can't use GTK stock here, as we hand these icons off 
+   to other processes in the case of notifications */
+#define ICON_PREFIX     DATA_DIR "/pixmaps/seahorse/48x48/"
+
 /* -----------------------------------------------------------------------------
  * PUBLIC METHODS
  */
@@ -72,7 +83,7 @@ process_crypto_result (SeahorsePGPOperation *pop, gpgme_error_t gstarterr,
         
         data = gpgme_data_release_and_get_mem (cryptdata, &len);
         *result = g_strndup (data, len);
-        free (data);
+        g_free (data);
         
         return TRUE;
         
@@ -88,6 +99,114 @@ process_crypto_result (SeahorsePGPOperation *pop, gpgme_error_t gstarterr,
     }
 }
 
+static gpgme_key_t* 
+keylist_to_keys (GList *keys)
+{
+	gpgme_key_t *recips;
+	int i;
+
+	recips = g_new0 (gpgme_key_t, g_list_length (keys) + 1);
+
+	for (i = 0; keys != NULL; keys = g_list_next (keys), i++) {
+		g_return_val_if_fail (SEAHORSE_IS_PGP_KEY (keys->data), recips);
+		recips[i] = SEAHORSE_PGP_KEY (keys->data)->pubkey;
+		gpgmex_key_ref (recips[i]);
+	}
+
+	return recips;
+}
+
+static void
+notify_signatures (const gchar* data, gpgme_verify_result_t status)
+{
+	const gchar *icon = NULL;
+	gchar *title, *body;
+	gboolean sig = FALSE;
+	GSList *rawids;
+	GList *keys;
+	SeahorseKey *skey;
+
+	/* Discover the key in question */
+	rawids = g_slist_append (NULL, status->signatures->fpr);
+	keys = seahorse_context_discover_objects (SCTX_APP (), SEAHORSE_PGP, rawids);
+	g_slist_free (rawids);
+
+	g_return_if_fail (keys != NULL);
+	skey = SEAHORSE_KEY (keys->data);
+	g_list_free (keys);
+
+	/* Figure out what to display */
+	switch (gpgme_err_code (status->signatures->status)) {
+	case GPG_ERR_KEY_EXPIRED:
+		/* TRANSLATORS: <key id='xxx'> is a custom markup tag, do not translate. */
+		body = _("Signed by <i><key id='%s'/> <b>expired</b></i> on %s.");
+		title = _("Invalid Signature");
+		icon = ICON_PREFIX "seahorse-sign-bad.png";
+		sig = TRUE;
+		break;
+		/* TRANSLATORS: <key id='xxx'> is a custom markup tag, do not translate. */
+	case GPG_ERR_SIG_EXPIRED:
+		body  = _("Signed by <i><key id='%s'/></i> on %s <b>Expired</b>.");
+		title = _("Expired Signature");
+		icon = ICON_PREFIX "seahorse-sign-bad.png";
+		sig = TRUE;
+		break;
+		/* TRANSLATORS: <key id='xxx'> is a custom markup tag, do not translate. */
+	case GPG_ERR_CERT_REVOKED:
+		body = _("Signed by <i><key id='%s'/> <b>Revoked</b></i> on %s.");
+		title = _("Revoked Signature");
+		icon = ICON_PREFIX "seahorse-sign-bad.png";
+		sig = TRUE;
+		break;
+	case GPG_ERR_NO_ERROR:
+		/* TRANSLATORS: <key id='xxx'> is a custom markup tag, do not translate. */
+		body = _("Signed by <i><key id='%s'/></i> on %s.");
+		title = _("Good Signature");
+		icon = ICON_PREFIX "seahorse-sign-ok.png";
+		sig = TRUE;
+		break;
+	case GPG_ERR_NO_PUBKEY:
+		body = _("Signing key not in keyring.");
+		title = _("Unknown Signature");
+		icon = ICON_PREFIX "seahorse-sign-unknown.png";
+		break;
+	case GPG_ERR_BAD_SIGNATURE:
+		body = _("Bad or forged signature. The signed data was modified.");
+		title = _("Bad Signature");
+		icon = ICON_PREFIX "seahorse-sign-bad.png";
+		break;
+	case GPG_ERR_NO_DATA:
+		return;
+	default:
+		if (!GPG_IS_OK (status->signatures->status))
+			seahorse_pgp_handle_gpgme_error (status->signatures->status, 
+			                            _("Couldn't verify signature."));
+		return;
+	};
+
+	if (sig) {
+		gchar *date = seahorse_util_get_display_date_string (status->signatures->timestamp);
+		gchar *id = seahorse_context_id_to_dbus (SCTX_APP (), seahorse_key_get_keyid (skey), 0);
+		body = g_markup_printf_escaped (body, id, date);
+		g_free (date);
+		g_free (id);
+	} else {
+		body = g_strdup (body);
+	}
+
+	if (data) {
+		data = seahorse_util_uri_get_last (data);
+		title = g_strdup_printf ("%s: %s", data, title);
+	} else {
+		title = g_strdup (title);
+	}
+
+	seahorse_notification_display (title, body, !sig, icon, NULL);
+
+	g_free(title);
+	g_free(body);
+}    
+
 /* -----------------------------------------------------------------------------
  * DBUS METHODS 
  */
@@ -100,8 +219,8 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
 {
     GList *recipkeys = NULL;
     SeahorsePGPOperation *pop; 
-    SeahorseKey *signkey = NULL;
-    SeahorseKey *skey;
+    SeahorseObject *signkey = NULL;
+    SeahorseObject *skey;
     gpgme_key_t *recips;
     gpgme_data_t plain, cipher;
     gpgme_error_t gerr;
@@ -114,7 +233,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
     
     /* The signer */
     if (signer && signer[0]) {
-        signkey = seahorse_context_key_from_dbus (SCTX_APP (), signer, NULL);
+        signkey = seahorse_context_object_from_dbus (SCTX_APP (), signer, NULL);
         if (!signkey) {
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
                          _("Invalid or unrecognized signer: %s"), signer);
@@ -122,7 +241,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
         }
         
         if (!SEAHORSE_IS_PGP_KEY (signkey) || 
-            !(seahorse_key_get_flags (signkey) & SKEY_FLAG_CAN_SIGN)) {
+            !(seahorse_object_get_flags (signkey) & SKEY_FLAG_CAN_SIGN)) {
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                          _("Key is not valid for signing: %s"), signer);
             return FALSE;
@@ -132,7 +251,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
     /* The recipients */
     for( ; recipients[0]; recipients++)
     {
-        skey = seahorse_context_key_from_dbus (SCTX_APP (), recipients[0], NULL);
+        skey = seahorse_context_object_from_dbus (SCTX_APP (), recipients[0], NULL);
         if (!skey) {
             g_list_free (recipkeys);
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
@@ -141,7 +260,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
         }
         
         if (!SEAHORSE_IS_PGP_KEY (skey) ||
-            !(seahorse_key_get_flags (skey) & SKEY_FLAG_CAN_ENCRYPT)) {
+            !(seahorse_object_get_flags (skey) & SKEY_FLAG_CAN_ENCRYPT)) {
             g_list_free (recipkeys);
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                          _("Key is not a valid recipient for encryption: %s"), recipients[0]);
@@ -171,13 +290,13 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
 
     /* Add the default key if set and necessary */
     if (seahorse_gconf_get_boolean (ENCRYPTSELF_KEY)) {
-        skey = seahorse_context_get_default_key (SCTX_APP ());
+        skey = SEAHORSE_OBJECT (seahorse_context_get_default_key (SCTX_APP ()));
         if (SEAHORSE_IS_PGP_KEY (skey))
             recipkeys = g_list_append (recipkeys, skey);
     }
 
     /* Make keys into the right format for GPGME */
-    recips = seahorse_util_keylist_to_keys (recipkeys);
+    recips = keylist_to_keys (recipkeys);
     g_list_free (recipkeys);
     
     /* Do the encryption */
@@ -190,7 +309,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
                                        plain, cipher);
     }
     
-    seahorse_util_free_keys (recips);
+    gpgmex_free_keys (recips);
 
     /* Frees cipher */
     ret = process_crypto_result (pop, gerr, cipher, crypttext, error);
@@ -205,7 +324,7 @@ seahorse_service_crypto_sign_text (SeahorseServiceCrypto *crypto, const char *si
                                    int flags, const char *cleartext, char **crypttext,
                                    GError **error)
 {
-    SeahorseKey *signkey = NULL;
+    SeahorseObject *signkey = NULL;
     gpgme_error_t gerr;
     SeahorsePGPOperation *pop; 
     gpgme_data_t plain, cipher;
@@ -221,7 +340,7 @@ seahorse_service_crypto_sign_text (SeahorseServiceCrypto *crypto, const char *si
         g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                      _("No signer specified"));
     
-    signkey = seahorse_context_key_from_dbus (SCTX_APP (), signer, NULL);
+    signkey = seahorse_context_object_from_dbus (SCTX_APP (), signer, NULL);
     if (!signkey) {
         g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
                      _("Invalid or unrecognized signer: %s"), signer);
@@ -229,7 +348,7 @@ seahorse_service_crypto_sign_text (SeahorseServiceCrypto *crypto, const char *si
     }
         
     if (!SEAHORSE_IS_PGP_KEY (signkey) || 
-        !(seahorse_key_get_flags (signkey) & SKEY_FLAG_CAN_SIGN)) {
+        !(seahorse_object_get_flags (signkey) & SKEY_FLAG_CAN_SIGN)) {
         g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                      _("Key is not valid for signing: %s"), signer);
         return FALSE;
@@ -272,7 +391,7 @@ seahorse_service_crypto_decrypt_text (SeahorseServiceCrypto *crypto,
     gboolean ret = TRUE;
     GQuark keyid;
     
-    if (!g_str_equal (ktype, g_quark_to_string (SKEY_PGP))) {
+    if (!g_str_equal (ktype, g_quark_to_string (SEAHORSE_PGP))) {
         g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                      _("Invalid key type for decryption: %s"), ktype);
         return FALSE;        
@@ -307,11 +426,12 @@ seahorse_service_crypto_decrypt_text (SeahorseServiceCrypto *crypto,
     
         if (status->signatures) {
             if (!(flags & FLAG_QUIET))
-                seahorse_notify_signatures (NULL, status);
+                notify_signatures (NULL, status);
             if (status->signatures->summary & GPGME_SIGSUM_GREEN ||
-                status->signatures->summary & GPGME_SIGSUM_VALID) {
+                status->signatures->summary & GPGME_SIGSUM_VALID ||
+                status->signatures->summary & GPGME_SIGSUM_KEY_MISSING) {
                 keyid = seahorse_pgp_key_get_cannonical_id (status->signatures->fpr);
-                *signer = seahorse_context_keyid_to_dbus (SCTX_APP (), keyid, 0);
+                *signer = seahorse_context_id_to_dbus (SCTX_APP (), keyid, 0);
             }
         }
     }
@@ -334,7 +454,7 @@ seahorse_service_crypto_verify_text (SeahorseServiceCrypto *crypto,
     gboolean ret = TRUE;
     GQuark keyid;
     
-    if (!g_str_equal (ktype, g_quark_to_string (SKEY_PGP))) {
+    if (!g_str_equal (ktype, g_quark_to_string (SEAHORSE_PGP))) {
         g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                      _("Invalid key type for verifying: %s"), ktype);
         return FALSE;        
@@ -369,11 +489,12 @@ seahorse_service_crypto_verify_text (SeahorseServiceCrypto *crypto,
     
         if (status->signatures) {
             if (!(flags & FLAG_QUIET))
-                seahorse_notify_signatures (NULL, status);
+                notify_signatures (NULL, status);
             if (status->signatures->summary & GPGME_SIGSUM_GREEN ||
-                status->signatures->summary & GPGME_SIGSUM_VALID) {
+                status->signatures->summary & GPGME_SIGSUM_VALID ||
+                status->signatures->summary & GPGME_SIGSUM_KEY_MISSING) {
                 keyid = seahorse_pgp_key_get_cannonical_id (status->signatures->fpr);
-                *signer = seahorse_context_keyid_to_dbus (SCTX_APP (), keyid, 0);
+                *signer = seahorse_context_id_to_dbus (SCTX_APP (), keyid, 0);
             }
         }
     }
